# WARNING: Please don't edit this file. It was generated by Python/WinRT v3.1.0

import enum

import winrt.system
from winrt._winrt_windows_media_dialprotocol import (
    DialApp,
    DialAppStateDetails,
    DialDevice,
    DialDevicePicker,
    DialDevicePickerFilter,
    DialDeviceSelectedEventArgs,
    DialDisconnectButtonClickedEventArgs,
    DialReceiverApp,
)

__all__ = [
    "DialAppLaunchResult",
    "DialAppState",
    "DialAppStopResult",
    "DialDeviceDisplayStatus",
    "DialApp",
    "DialAppStateDetails",
    "DialDevice",
    "DialDevicePicker",
    "DialDevicePickerFilter",
    "DialDeviceSelectedEventArgs",
    "DialDisconnectButtonClickedEventArgs",
    "DialReceiverApp",
]

class DialAppLaunchResult(enum.IntEnum):
    LAUNCHED = 0
    FAILED_TO_LAUNCH = 1
    NOT_FOUND = 2
    NETWORK_FAILURE = 3

class DialAppState(enum.IntEnum):
    UNKNOWN = 0
    STOPPED = 1
    RUNNING = 2
    NETWORK_FAILURE = 3

class DialAppStopResult(enum.IntEnum):
    STOPPED = 0
    STOP_FAILED = 1
    OPERATION_NOT_SUPPORTED = 2
    NETWORK_FAILURE = 3

class DialDeviceDisplayStatus(enum.IntEnum):
    NONE = 0
    CONNECTING = 1
    CONNECTED = 2
    DISCONNECTING = 3
    DISCONNECTED = 4
    ERROR = 5

